#include "Fader.h"
#include "Constants.h"
#include "SDL/SDL.h"
#include "Globals.h"
#include "Functions.h"

void cFader::fadeOut(int pDuration, Uint32 fadeColor) {
	cFader::duration = pDuration;
	cFader::count = 0;
	cFader::endColor = fadeColor;
	cFader::status = FADE_ATC;

	if (cFader::currentColor != fadeColor) {
		SDL_FillRect(cFader::fadeSurface, NULL, fadeColor);
		cFader::currentColor = fadeColor;
	}
}

void cFader::fadeIn(int pDuration, Uint32 fadeColor) {
	cFader::duration = pDuration;
	cFader::count = 0;
	cFader::endColor = fadeColor;
	cFader::status = FADE_CTA;

	if (cFader::currentColor != fadeColor) {
		SDL_FillRect(cFader::fadeSurface, NULL, fadeColor);
		cFader::currentColor = fadeColor;
	}
}

int cFader::fadeRender() {
	int alpha;

	switch (cFader::status) {
		case FADE_CTA:
			alpha = 255 - (cFader::count / (float)cFader::duration) * 255;
			cFader::count++;

			SDL_SetAlpha(cFader::fadeSurface, SDL_SRCALPHA, alpha);
			applySurface(0, 0, cFader::fadeSurface);

			if (cFader::count == cFader::duration) {cFader::status = FADE_A;}
			break;
		case FADE_ATC:
			alpha = (cFader::count / (float)cFader::duration) * 255;
			cFader::count++;

			SDL_SetAlpha(cFader::fadeSurface, SDL_SRCALPHA, alpha);
			applySurface(0, 0, cFader::fadeSurface);

			if (cFader::count == cFader::duration) {cFader::status = FADE_C;}
			break;
		case FADE_C:
			applySurface(0, 0, cFader::fadeSurface);
	}

	// applySurface(0, 0, cFader::fadeSurface);

	return cFader::status;
}

void cFader::prepareFade() {
	SDL_Surface* tSurface;

	Uint32 rmask, gmask, bmask;
	#if SDL_BYTEORDER == SDL_BIG_ENDIAN
    rmask = 0xff000000;
    gmask = 0x00ff0000;
    bmask = 0x0000ff00;
	#else
    rmask = 0x000000ff;
    gmask = 0x0000ff00;
    bmask = 0x00ff0000;
	#endif

	tSurface = SDL_CreateRGBSurface(SDL_SWSURFACE | SDL_SRCALPHA, SCR_WIDTH, SCR_HEIGHT, SCR_BPP, rmask, gmask, bmask, 0);

	// SDL_SetAlpha(tSurface, SDL_SRCALPHA, 255);
	SDL_FillRect(tSurface, NULL, cFader::standardColor);

	cFader::fadeSurface = SDL_DisplayFormat(tSurface);
	SDL_FreeSurface(tSurface);
}

cFader::cFader(Uint32 pStandardColor) {
	cFader::standardColor = pStandardColor;
	cFader::fadeSurface = NULL;
	cFader::currentColor = pStandardColor;
}
