#include "SDL/SDL.h"
#include "SDL/SDL_ttf.h"
#include "SDL/SDL_image.h"
#include "math.h"
#include <string>
#include <sstream>

#include "Functions.h"
#include "Globals.h"
#include "Constants.h"
#include "Colors.h"
#include "bFuncs.h"

void makeRect(int x, int y, int width, int height, int border, Uint32 borderColor, Uint32 midColor) {
	SDL_Rect t;
	int iX, iY, iW, iH;
	int botBorderY;
	int rightBorderX;
	int middleBorderY;

	iX = x + border;
	iY = y + border;
	iW = width - 2 * border;
	iH = height - 2 * border;

	botBorderY = y + border + iH;
	rightBorderX = x + border + iW;
	middleBorderY = y + border;

	// Top Border
	t.x = x; t.y = y; t.w = width; t.h = border; SDL_FillRect(screen, &t, borderColor);
	// Bottom Border
	t.x = x; t.y = botBorderY; t.w = width; t.h = border; SDL_FillRect(screen, &t, borderColor);
	// Left Border
	t.x = x; t.y = middleBorderY; t.w = border; t.h = iH; SDL_FillRect(screen, &t, borderColor);
	// Right Border
	t.x = rightBorderX; t.y = middleBorderY; t.w = border; t.h = iH; SDL_FillRect(screen, &t, borderColor);

	// Center
	t.x = iX; t.y = iY; t.w = iW; t.h = iH; SDL_FillRect(screen, &t, midColor);

}

// TTF_RenderText_Shaded?

int button(std::string caption, int x, int y, int mX, int mY, bool buttonDown, TTF_Font* vont, bool leftMouseButton) {
	// Define some colors & the text surface
	SDL_Color cRed = {255, 0, 0};
	SDL_Color cOrange = {255, 165, 0};
	SDL_Color cDarkGreen = {0, 100, 0};
	SDL_Surface* tSurface;

	// Render the caption
	int returnVal = BUTTON_NOT;
	tSurface = TTF_RenderText_Solid(vont, caption.c_str(), cRed);

	int xM = x - tSurface->w;

	// Check if the mouse is in range
	if (mX >= xM && mY >= y && mY <= y + tSurface->h && mX <= xM + tSurface->w) {
		if (buttonDown) {
			// Set the right color
			SDL_SetColors(tSurface, &cDarkGreen, 1, 256);
			applySurface(xM, y, tSurface);
			returnVal = BUTTON_CLICKD;
		} else if (leftMouseButton) {
			SDL_SetColors(tSurface, &cDarkGreen, 1, 256);
			applySurface(xM, y, tSurface);
			returnVal = BUTTON_OVER;
		} else {
			SDL_SetColors(tSurface, &cOrange, 1, 256);
			applySurface(xM, y, tSurface);
			returnVal = BUTTON_OVER;
		}
	} else {
		applySurface(xM, y, tSurface);
	}

	SDL_FreeSurface(tSurface);

	return returnVal;
}

int applySurface(int x, int y, SDL_Surface* src) {
	SDL_Rect tt;
	tt.x = x;
	tt.y = y;

	if (SDL_BlitSurface(src, NULL, screen, &tt) < 0) {
		return 1;
	}

	return 0;
}

int applySurfaceSprite(int x, int y, SDL_Surface* src, SDL_Surface* dst, SDL_Rect spriteRect) {
	SDL_Rect tt;
	tt.x = x;
	tt.y = y;

	if (SDL_BlitSurface(src, &spriteRect, dst, &tt) < 0) {
		return 1;
	}

	return 0;
}

int applySurfaceMid(int x, int y, SDL_Surface* src, SDL_Surface* dst) {
	SDL_Rect tt;
	tt.x = x - src->w / 2;
	tt.y = y - src->h / 2;

	if (SDL_BlitSurface(src, NULL, dst, &tt) < 0) {
		return 1;
	}

	return 0;
}

SDL_Surface* loadImage(std::string filename) {
	SDL_Surface* loaded = NULL;
	SDL_Surface* optimized = NULL;

	loaded = IMG_Load(filename.c_str());
	if (loaded != NULL) {
		optimized = SDL_DisplayFormatAlpha(loaded);

		SDL_FreeSurface(loaded);
	}

	return optimized;
}

int regulateFPS(int timeS, int timeE) {
	int timeDiff = timeE - timeS;

	// Maximum tijd dat het programma over een frame mag doen
	// Time Per Frame
	int maxTPF = 1000 / SCR_FPS;

	// Als tijd die programma over het frame deed korter is dan het maximum TPF
	// Het programma laten pauzeren voor het verschil tussen die twee tijden.
	if (timeDiff < maxTPF) {
		SDL_Delay(maxTPF - timeDiff);
	}

	return timeDiff;
}

int initButtons() {
	bPlay.menuState = STATE_IN;
	bHelp.menuState = STATE_IN;
	bAbout.menuState = STATE_IN;
	bExit.menuState = STATE_IN;

	bEasy.menuState = STATE_AWAY;
	bNormal.menuState = STATE_AWAY;
	bHard.menuState = STATE_AWAY;
	bBack_Play.menuState = STATE_AWAY;

	bResume.menuState = STATE_AWAY;
	bBack_Menu.menuState = STATE_AWAY;
	bExit_Game.menuState = STATE_AWAY;

	SDL_Color textColor = {255, 0, 0};

	if (bPlay.renderText(textColor) == 1) {return 1;}
	if (bHelp.renderText(textColor) == 1) {return 1;}
	if (bAbout.renderText(textColor) == 1) {return 1;}
	if (bExit.renderText(textColor) == 1) {return 1;}

	if (bEasy.renderText(textColor) == 1) {return 1;}
	if (bNormal.renderText(textColor) == 1) {return 1;}
	if (bHard.renderText(textColor) == 1) {return 1;}
	if (bBack_Play.renderText(textColor) == 1) {return 1;}

	if (bResume.renderText(textColor) == 1) {return 1;}
	if (bBack_Menu.renderText(textColor) == 1) {return 1;}
	if (bExit_Game.renderText(textColor) == 1) {return 1;}

	return 0;
}

void renderBackground(int* bVx, int* bVy, int* horDir, int* verDir) {
	int top = *bVy;
	int left = *bVx;

	left += *horDir;
	top += *verDir;

	int bot = top + 600;
	int right = left + 800;

	if (left <= 0) {
		left = 0;
		*horDir = 1;
	} else if (right >= 1920) {
		right = 1920;
		left = right - 800;
		*horDir = -1;
	}

	if (top <= 0) {
		top = 0;
		*verDir = 1;
	} else if (bot >= 1920) {
		bot = 1920;
		top = bot - 600;
		*verDir = -1;
	}

	*bVx = left;
	*bVy = top;

	applySurface(-1 * (*bVx), -1 * (*bVy), background);
}

void updateButtons(int x, int y, Uint8 mouseState) {
	bPlay.render(mouseState, x, y, &buttonPlay);
	bHelp.render(mouseState, x, y, &buttonHelp);
	bAbout.render(mouseState, x, y, &buttonAbout);
	bExit.render(mouseState, x, y, &buttonExit);

	bEasy.render(mouseState, x, y, &buttonEasy);
	bNormal.render(mouseState, x, y, &buttonNormal);
	bHard.render(mouseState, x, y, &buttonHard);
	bBack_Play.render(mouseState, x, y, &buttonBack_Play);
}

void renderMenu(int tX, int tY, Uint8 mouseState, int* bVx, int* bVy, int* horDir, int* verDir) {
	renderBackground(bVx, bVy, horDir, verDir);

	applySurface(0, 0, logo);
	applySurface(245, 2, multiplayer);

	updateButtons(tX, tY, SDL_GetMouseState(&tX, &tY));

	if (menuExit) {
		if (!bPlay.visible()) {
			loop = false;
		}
	}
}

int loadPictures() {
	std::stringstream picName;
	for (int i = 1; i <= 32; i++) {
		picName << "Media/Images/Cards/image" << i << ".png";
		memPics[i-1] = loadImage(picName.str().c_str());
		picName.str("");
		if (memPics[i-1] == NULL) {
			return 1;
		}
	}
	return 0;
}

int getIndex(int x, int y, int w) {
	return y * w + x;
}
void balk(int x, int y, int w, int h, Uint32 clr) {
	SDL_Rect lt;
	lt.x = x;
	lt.y = y;
	lt.w = w;
	lt.h = h;

	SDL_FillRect(screen, &lt, clr);
}

// http://130.113.54.154/~monger/hsl-rgb.html

int HSLtoRGB(float h, float s, float l) {
	float t1;
	float t2;
	float t3r, t3g, t3b;
	float r, g, b;
	int rF, gF, bF;
	int clr;

	if (s == 0) {
		r = l * 100;
		g = l * 100;
		b = l * 100;

		clr = SDL_MapRGB(screen->format, r, g, b);
		return clr;
	}

	if (l < 0.5) {
		t2 = l * (1.0 + s);
	} else if (l >= 0.5) {
		t2 = l + s - l * s;
	}

	t1 = 2.0 * l - t2;

	t3r = h + 1.0/3.0;
	if (t3r < 0) {t3r += 1.0;}
	if (t3r > 1) {t3r -= 1.0;}

	t3g = h;
	if (t3g < 0) {t3g += 1.0;}
	if (t3g > 1) {t3g -= 1.0;}

	t3b = h - 1.0/3.0;
	if (t3b < 0) {t3b += 1.0;}
	if (t3b > 1) {t3b -= 1.0;}

	if (6.0 * t3r < 1) {r = t1 + (t2 - t1) * 6.0 * t3r;}
	else if (2.0 * t3r < 1) {r = t2;}
	else if (3.0 * t3r < 2) {r = t1 + (t2 - t1) * ((2.0 / 3.0) - t3r) * 6.0;}
	else {r = t1;}

	if (6.0 * t3g < 1) {g = t1 + (t2 - t1) * 6.0 * t3g;}
	else if (2.0 * t3g < 1) {g = t2;}
	else if (3.0 * t3g < 2) {g = t1 + (t2 - t1) * ((2.0 / 3.0) - t3g) * 6.0;}
	else {g = t1;}

	if (6.0 * t3b < 1) {b = t1 + (t2 - t1) * 6.0 * t3b;}
	else if (2.0 * t3b < 1) {b = t2;}
	else if (3.0 * t3b < 2) {b = t1 + (t2 - t1) * ((2.0 / 3.0) - t3b) * 6.0;}
	else {b = t1;}

	rF = r * 255;
	gF = g * 255;
	bF = b * 255;

	clr = SDL_MapRGB(screen->format, rF, gF, bF);

	return clr;
}

void displayScore(int p1, int p2) {
	SDL_Color cRed = {255, 0, 0};
	SDL_Color cWhite = {255, 255, 255};

	SDL_Surface* surfP1 = NULL;
	SDL_Surface* surfP2 = NULL;
	SDL_Surface* headP1 = NULL;
	SDL_Surface* headP2 = NULL;
	SDL_Surface* caption = NULL;

	// Stringstream to convert numbers
	std::stringstream ss;

	ss << p1;

	surfP1 = TTF_RenderText_Shaded(font3, ss.str().c_str(), cRed, cWhite);

	ss.str("");
	ss << p2;

	surfP2 = TTF_RenderText_Shaded(font3, ss.str().c_str(), cRed, cWhite);

	headP1 = TTF_RenderText_Shaded(font2, "Player 1", cRed, cWhite);
	headP2 = TTF_RenderText_Shaded(font2, "Player 2", cRed, cWhite);
	caption = TTF_RenderText_Shaded(font, "Stats", cRed, cWhite);

	// Scores
	applySurface(0, 550, surfP1);
	applySurface(180 - surfP2->w, 550, surfP2);

	// Captions
	applySurface(0, 530, headP1);
	applySurface(180 - headP2->w, 530, headP2);
	applySurface(180-caption->w, 504 - caption->h * .5, caption);

	SDL_FreeSurface(surfP1);
	SDL_FreeSurface(surfP2);
	SDL_FreeSurface(headP1);
	SDL_FreeSurface(headP2);
	SDL_FreeSurface(caption);
}

float cardBackground(float degMod, bool disabled) {
	SDL_Rect tr;
	tr.x = 190;
	tr.y = 0;
	tr.w = 610;
	tr.h = 600;

	tr.x = 180;
	tr.y = 0;
	tr.w = 620;
	tr.h = 600;

	SDL_FillRect(screen, &tr, SDL_MapRGB(screen->format, 0, 191, 255));



	/*
	for (int x = 190; x < 800; x++) {
		step = x - 190;
		y = cos((((x / 610.0) * 360.0) - 20 + degMod) * (3.14159265 / 180.0)) * 300.0 + 300.0;

		SDL_LockSurface(screen);
		for (int mod = 0; mod < 101; mod++) {
			y = y - mod;
			check = true;

			if (y > 599) {
				check = false;
			}
			if (y < 0) {
				check = false;
			}

			if (check) {
				putpixel(screen, x, y, HSLtoRGB(awesomeColor, 0.5, 0.5));
			}
		}
		for (int mod = 0; mod < 101; mod++) {
			y = y + mod;
			check = true;

			if (y > 599) {
				check = false;
			}
			if (y < 0) {
				check = false;
			}

			if (check) {
				putpixel(screen, x, y, HSLtoRGB(awesomeColor, 0.5, 0.5));
			}
		}
		SDL_UnlockSurface(screen);
	}

	if (!disabled) {degMod += 1;}
	*/

	return degMod;
}

void putpixel(SDL_Surface *surface, int x, int y, Uint32 pixel)
{
    int bpp = surface->format->BytesPerPixel;
    /* Here p is the address to the pixel we want to set */
    Uint8 *p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;

    switch(bpp) {
    case 1:
        *p = pixel;
        break;

    case 2:
        *(Uint16 *)p = pixel;
        break;

    case 3:
        if(SDL_BYTEORDER == SDL_BIG_ENDIAN) {
            p[0] = (pixel >> 16) & 0xff;
            p[1] = (pixel >> 8) & 0xff;
            p[2] = pixel & 0xff;
        } else {
            p[0] = pixel & 0xff;
            p[1] = (pixel >> 8) & 0xff;
            p[2] = (pixel >> 16) & 0xff;
        }
        break;

    case 4:
        *(Uint32 *)p = pixel;
        break;
    }
}
