/*
Custard Memory Project
Begonnen op 31/03/2011

X = Ruimte tussen plaatjes
Y = Breedte plaatjes
600 = 9 * X + 8 * Y

Toevoegen:
PAUSE plaatje in pauze scherm
Terug naar menu knop in help

*/

#include "SDL/SDL.h"
#include "SDL/SDL_ttf.h"
#include "SDL/SDL_image.h"
#include <string>
#include <sstream>
#include "time.h"

#include "Globals.h"
#include "Functions.h"
#include "Constants.h"
#include "bFuncs.h"
#include "Button.h"
#include "Fader.h"
#include "Memory.h"

int main( int argc, char* args[] )
{
	int a;
	a = Init();
	if (a > 0) {
		return a;
	}

	if (Game() > 0) {
		return 2;
	}

	Exit();

    return 0;
}

int Init() {
	// Systemen opstarten en plaatjes genereren/laden

	srand(time(NULL));

	if (SDL_Init( SDL_INIT_TIMER | SDL_INIT_VIDEO | SDL_INIT_AUDIO ) == -1) {
		return 1;
	}

	SDL_WM_SetCaption("Custard Memory (C)", NULL);
	SDL_WM_SetIcon(SDL_LoadBMP("Media/favicon.bmp"), NULL);

	screen = SDL_SetVideoMode(SCR_WIDTH, SCR_HEIGHT, SCR_BPP, SDL_SWSURFACE);
	if (screen == NULL) {
		return 1;
	}

	if (TTF_Init() == -1) {
		return 1;
	}

	font = TTF_OpenFont("Media/arial.ttf", 30);
	if (font == NULL) {
		return 1;
	}

	font2 = TTF_OpenFont("Media/arial.ttf", 20);
	if (font2 == NULL) {
		return 1;
	}

	font3 = TTF_OpenFont("Media/arial.ttf", 40);
	if (font3 == NULL) {
		return 1;
	}

	background = loadImage("Media/Images/background.png");
	if (background == NULL) {
		return 1;
	}

	logo = loadImage("Media/Images/Logo.png");
	if (logo == NULL) {
		return 1;
	}

	arrows = loadImage("Media/Images/arrows.png");
	if (arrows == NULL) {
		return 1;
	}

	std::stringstream t;
	SDL_Color cWhite = {255, 0, 0};
	t << 1;
	one = TTF_RenderText_Blended(font, t.str().c_str(), cWhite);
	if (one == NULL) {
		return 1;
	}
	t.str("");

	t << 2;
	two = TTF_RenderText_Blended(font, t.str().c_str(), cWhite);
	if (two == NULL) {
		return 1;
	}

	firstPic = TTF_RenderText_Blended(font, "First card", cWhite);
	if (firstPic == NULL) {
		return 1;
	}

	secondPic = TTF_RenderText_Blended(font, "Second card", cWhite);
	if (secondPic == NULL) {
		return 1;
	}

	v = loadImage("Media/Images/v.png");
	if (v == NULL) {
		return 1;
	}

	x = loadImage("Media/Images/x.png");
	if (x == NULL) {
		return 1;
	}

	multiplayer = loadImage("Media/Images/2players.png");
	if (multiplayer == NULL) {
		return 1;
	}

	p1Wins = loadImage("Media/Images/p1wins.png");
	if (p1Wins == NULL) {
		return 1;
	}

	p2Wins = loadImage("Media/Images/p2wins.png");
	if (p2Wins == NULL) {
		return 1;
	}

	tied = loadImage("Media/Images/tied.png");
	if (tied == NULL) {
		return 1;
	}

	kaartje = loadImage("Media/Images/kaartje.png");
	if (kaartje == NULL) {
		return 1;
	}

	cardBackgroundSurf = loadImage("Media/Images/cardbackground.png");
	if (cardBackgroundSurf == NULL) {
		return 1;
	}

	cardBGRanden = loadImage("Media/Images/randen.png");
	if (cardBGRanden == NULL) {
		return 8;
	}

	paused = loadImage("Media/Images/pause.png");
	if (paused == NULL) {
		return 1;
	}

	about = loadImage("Media/Images/about.png");
	if (about == NULL) {
		return 1;
	}

	t.str("");
	for (int i = 0; i < HELP_MAX; i++) {
		t << "Media/Images/Help/p" << i+1 << ".png";
		helpPics[i] = NULL;
		helpPics[i] = loadImage(t.str().c_str());
		if (helpPics[i] == NULL) {
			return i+1;
		}
		t.str("");
	}

	initButtons();
	fader.prepareFade();

	if (loadPictures() > 0) {
		return 7;
	}

	return 0;
}

int Game() {
	int tX, tY;
	int bVx = 640, bVy = 640;
	int horDir = 1, verDir = 1;
	int FPS = 60, timeDiff = 0;
	int fadeState = 0;
	int helpCount = 0;
	bool prevLMB = false;
	Uint8 mouseState;
	Uint8* keyState;
	std::stringstream tS;

	float aboutStep = 267 / (float)SCR_FPS * 3;
	float aboutMargin = -267 - about->h;

	// float xStep = (SCR_WIDTH - 200) / 8.0, yStep = SCR_HEIGHT / 8.0;

	fader.fadeIn(SCR_FPS * 0.8, SDL_MapRGB(screen->format, 0, 0, 0));

	mouseState = SDL_GetMouseState(&tX, &tY);

	while (loop) {
		// FPS Var
		timerStart = SDL_GetTicks();
		keyState = SDL_GetKeyState(NULL);

		while (SDL_PollEvent(&event)) {
			switch (event.type) {
				case SDL_QUIT:
					loop = false;
					break;
				case SDL_KEYDOWN:
					// Ingame menu laten zien wanneer escape ingedrukt wordt
					if (event.key.keysym.sym == SDLK_ESCAPE && memoryVisible) {
						if (memory.menuStatus) {
							// Make them fly away when the menu is visible
							if (bResume.menuState != STATE_OUT) {
								memory.menuStatus = false;

								bResume.menuState = STATE_OUT;
								bBack_Menu.menuState = STATE_OUT;
								bExit_Game.menuState = STATE_OUT;
							}
						} else {
							// Make them fly in when the menu isn't
							if (bResume.menuState != STATE_OUT) {
								memory.menuStatus = true;

								bResume.menuState = STATE_IN;
								bBack_Menu.menuState = STATE_IN;
								bExit_Game.menuState = STATE_IN;
							}
						}
					} else if (event.key.keysym.sym == SDLK_TAB && memoryVisible) {
						// Nope
					}
					break;
			}
		}

		// Get Mouse Status
		prevLMB = mouseState & SDL_BUTTON(SDL_BUTTON_LEFT);
		mouseState = SDL_GetMouseState(&tX, &tY);

		// RENDER THE MENU WHOOP WHOOP
		if (menuVisible) {
			renderMenu(tX, tY, mouseState, &bVx, &bVy, &horDir, &verDir);

			if (aboutVisible) {
				if (aboutState == STATE_IN) {
					aboutMargin += aboutStep;
					if (aboutMargin > 0) {
						aboutMargin = 0;
						aboutState = STATE_MENU;
					}
				}

				if (aboutState == STATE_OUT) {
					aboutMargin += aboutStep;
					if (aboutMargin + 267 >= SCR_HEIGHT) {
						aboutMargin = -267 - about->h;
						aboutState = STATE_AWAY;
						aboutVisible = false;
					}
				}

				if (aboutState != STATE_AWAY) {
					applySurface(33, 267 + aboutMargin, about);
				}
			}
		}

		// Memory renderen als het bezig is.
		if (memoryVisible) {
			memory.renderMemory(tX, tY, mouseState);
		}

		// Helpscherm doen
		if (helpVisible) {
			if (helpCount < 0) {helpCount = 0;}
			if (helpCount >= HELP_MAX) {helpCount = 6;}

			applySurface(0, 0, helpPics[helpCount]);

			balk(220, 510, 313, 33, 0);

			if (helpCount > 0) {
				if (button("Previous", 340, 510, tX, tY, !prevLMB & (mouseState & SDL_BUTTON(SDL_BUTTON_LEFT)), font, SDL_BUTTON(SDL_BUTTON_LEFT) & mouseState) == BUTTON_CLICKD) {
					helpCount--;
				}
			}

			if (helpCount < HELP_MAX - 1) {
				if (button("Next", 530, 510, tX, tY, !prevLMB & (mouseState & SDL_BUTTON(SDL_BUTTON_LEFT)), font, SDL_BUTTON(SDL_BUTTON_LEFT) & mouseState) == BUTTON_CLICKD) {
					helpCount++;
				}
			}

			if (button("Back", 435, 510, tX, tY, !prevLMB & (mouseState & SDL_BUTTON(SDL_BUTTON_LEFT)), font, SDL_BUTTON(SDL_BUTTON_LEFT) & mouseState) == BUTTON_CLICKD) {
				fader.fadeOut(SCR_FPS * 2, 0);
				toMenu = true;
			}
		}

		// Check for closing program when the buttons are away and the Exit to Windows button was pressed.
		if (toWindows) {
			if (bResume.menuState == STATE_AWAY) {
				loop = false;
			}
		}

		// Fade everything in and out properly
		fadeState = fader.fadeRender();

		// Check whether the menu should be visible or not.
		// When it's fading to black & it's going to a game, the menu is not drawn.
		// When it's fading to transparent & it's going to the menu, the menu is drawn.
		if (fadeState == FADE_C && toGame) {
			menuVisible = false;
			memoryVisible = true;

			toMenu = false;
			toGame = false;
			toHelp = false;

			memory.startMemory(gamemode);

			fader.fadeIn(SCR_FPS * 2, 0);
		} else if (fadeState == FADE_C && toMenu) {
			menuVisible = true;
			memoryVisible = false;
			helpVisible = false;

			toGame = false;
			toMenu = false;
			toHelp = false;

			bPlay.menuState = STATE_IN;
			bHelp.menuState = STATE_IN;
			bAbout.menuState = STATE_IN;
			bExit.menuState = STATE_IN;

			helpCount = 0;

			fader.fadeIn(SCR_FPS * 2, 0);
		} else if (fadeState == FADE_C && toHelp) {
			menuVisible = false;
			helpVisible = true;

			toGame = false;
			toMenu = false;
			toHelp = false;

			fader.fadeIn(SCR_FPS * 2, 0);
		}

		// Check for TAB cheat
		/*
		if (keyState[SDLK_TAB] & memoryVisible) {
			for (int iX = 0; iX < 8; iX++) {
				for (int iY = 0; iY < 8; iY++) {
					if (memory.field[iX][iY].enabled) {
						tS << memory.field[iX][iY].contains;
						numSurf = TTF_RenderText_Solid(font2, tS.str().c_str(), cRed);
						tS.str("");
						applySurfaceMid(200 + (iX * xStep + xStep * .5), iY * yStep + yStep * .5, numSurf, screen);
						SDL_FreeSurface(numSurf);
					}
				}
			}
		}
		*/

		// Standard Game stuff (FPS etc.)
		// Sync & Clear the screen for the next frame
		SDL_Flip(screen);
		SDL_FillRect(screen, NULL, SDL_MapRGB(screen->format, 255, 255, 255));

		// FPS Calculation & Display
		timerEnd = SDL_GetTicks();

		regulateFPS(timerStart, timerEnd);

		timerREnd = SDL_GetTicks();
		timeDiff = timerREnd - timerStart;
		FPS = 1000 / timeDiff;

		std::stringstream windowTitle;
		windowTitle << "Custard Memory (C)" << " | " << FPS << " Frames per Second" << " | (" << tX << ", " << tY << ") " << aboutState << " | " << aboutMargin;
		SDL_WM_SetCaption(windowTitle.str().c_str(), NULL);

		awesomeColor += 0.001;
		if (awesomeColor > 1) {
			awesomeColor = 0;
		}
	}

	return 0;
}

int Exit() {
	SDL_FreeSurface(bPlay.captionSurf);
	SDL_FreeSurface(bHelp.captionSurf);
	SDL_FreeSurface(bAbout.captionSurf);
	SDL_FreeSurface(bExit.captionSurf);

	SDL_FreeSurface(bEasy.captionSurf);
	SDL_FreeSurface(bNormal.captionSurf);
	SDL_FreeSurface(bHard.captionSurf);
	SDL_FreeSurface(bBack_Play.captionSurf);

	SDL_FreeSurface(bResume.captionSurf);
	SDL_FreeSurface(bBack_Menu.captionSurf);
	SDL_FreeSurface(bExit_Game.captionSurf);

	SDL_FreeSurface(one);
	SDL_FreeSurface(two);
	SDL_FreeSurface(firstPic);
	SDL_FreeSurface(secondPic);
	SDL_FreeSurface(background);
	SDL_FreeSurface(logo);
	SDL_FreeSurface(v);
	SDL_FreeSurface(x);
	SDL_FreeSurface(arrows);
	SDL_FreeSurface(multiplayer);
	SDL_FreeSurface(p1Wins);
	SDL_FreeSurface(p2Wins);
	SDL_FreeSurface(tied);
	SDL_FreeSurface(kaartje);
	SDL_FreeSurface(cardBackgroundSurf);
	SDL_FreeSurface(cardBGRanden);
	SDL_FreeSurface(paused);
	SDL_FreeSurface(about);

	SDL_FreeSurface(fader.fadeSurface);

	for (int i = 0; i <= 31; i++) {
		SDL_FreeSurface(memPics[i]);
	}

	for (int i = 0; i < 7; i++) {
		SDL_FreeSurface(helpPics[i]);
	}

	TTF_CloseFont(font);

	TTF_Quit();

    SDL_Quit();

	return 0;
}

void func() {}
