#include "Functions.h"
#include "Button.h"
#include "Constants.h"
#include "Globals.h"
#include <string>
#include "SDL.h"
#include "SDL_ttf.h"


cButton::cButton (int x, int y, int w, int h, int border, Uint32 bColor, Uint32 mColor, std::string bCaption, bool en) {
	cButton::area.x = x;
	cButton::area.y = y;
	cButton::area.w = w;
	cButton::area.h = h;

	cButton::bColor = bColor;
	cButton::mColor = mColor;
	cButton::border = border;
	
	cButton::enabled = en;
	cButton::margin = SCR_WIDTH - x;
	cButton::marginStep = -1 * ((SCR_WIDTH - BUTTON_XPOS) / (SCR_FPS / 4.0));
	cButton::menuState = STATE_IN;
	cButton::captionStr = bCaption;
	cButton::captionSurf = NULL;
}

int cButton::render(Uint8 mouseState, int mX, int mY, void(*p)()) {
	// Save the previous state of the button, and quit the function if
	// The button isn't enabled
	cButton::prevState = cButton::state;
	cButton::state = BUTTON_NOT;
	if (cButton::enabled == false) {return 0;}
	
	// Change the state of the button to whether the button is clicked or hovered over
	if (mX >= area.x + margin && mX <= area.x + margin + area.w && mY >= area.y && mY < area.y + area.h) {
		cButton::state = BUTTON_OVER;

		if (mouseState & SDL_BUTTON(SDL_BUTTON_LEFT) && cButton::menuState == STATE_MENU) {
			cButton::state = BUTTON_CLICKD;
		}
	}
	
	// Calculating some colors
	Uint8 rB, gB, bB;
	Uint8 rM, gM, bM;
	SDL_GetRGB(bColor, screen->format, &rB, &gB, &bB);
	SDL_GetRGB(mColor, screen->format, &rM, &gM, &bM);
	rB = 255 - rB;
	gB = 255 - gB;
	bB = 255 - bB;
	rM = 255 - rM;
	gM = 255 - gM;
	bM = 255 - bM;

	// Make the button move in or move out of the screen depending on its state.
	// - STATE_AWAY: Button is out of the screen
	// - STATE_MENU: Button is at it's given position
	// - STATE_IN: Button is travelling to it's given position
	// - STATE_OUT: Button is travelling away from it's position
	if (menuState != STATE_MENU && menuState != STATE_AWAY) {
		margin = margin + marginStep;
	}

	if (menuState == STATE_IN) {
		if (margin <= 0) {
			menuState = STATE_MENU;
		}
	}

	if (menuState == STATE_OUT) {
		if (margin <= -1 * (area.x + area.w)) {
			margin = SCR_WIDTH - area.x + 10;
			menuState = STATE_AWAY;
		}
	}
	
	// Draw the right button depending on it's state
	switch (cButton::state) {
		case BUTTON_CLICKD:
			makeRect(area.x + margin, area.y, area.w, area.h, border, SDL_MapRGB(screen->format, rB * .5, gB * .5, bB * .5), SDL_MapRGB(screen->format, rM * .5, gM * .5, bM * .5));
			break;
		case BUTTON_OVER:
			makeRect(area.x + margin, area.y, area.w, area.h, border, SDL_MapRGB(screen->format, rB, gB, bB), SDL_MapRGB(screen->format, rM, gM, bM));
			break;
		case BUTTON_NOT:
			makeRect(area.x + margin, area.y, area.w, area.h, border, bColor, mColor);
			break;
	}
	
	// Calculate middle of the button and apply the button caption
	int tX, tY;
	tX = area.x + area.w / 2 - cButton::captionSurf->w / 2;
	tY = area.y + area.h / 2 - cButton::captionSurf->h / 2;
	
	if (captionSurf != NULL) {
		applySurface(tX + margin, tY, captionSurf);
	}
	
	// Execute the function that belongs with this button wanneer muisknop losgelaten wordt
	if (cButton::state == BUTTON_OVER && cButton::prevState == BUTTON_CLICKD) {
		p();
	}

	return 0;
}

int cButton::renderText(SDL_Color textColor) {
	// SDL_Color tColor = {255, 0, 0};
	cButton::captionSurf = TTF_RenderText_Blended(font, captionStr.c_str(), textColor);

	if (cButton::captionSurf == NULL) {
		return 1;
	}

	return 0;
}

bool cButton::visible() {
	if (area.x + margin >= SCR_WIDTH || area.x + margin + area.w <= 0) {
		return false;
	}

	return true;
}