#include "SDL.h"
#include "math.h"
#include "Constants.h"
#include "Functions.h"
#include "Globals.h"
#include "Memory.h"
#include "memPics.h"
#include "bFuncs.h"
#include "Colors.h"


cMemory::cMemory(int what) {
	// De waarden voor Memory vaststellen. O.a. kleuren vakjes, groottes vakjes,
	// zorgen dat ze allemaal uitstaan en niet geselecteerd zijn.

	float xStep = (SCR_WIDTH - 200) / 8.0, yStep = SCR_HEIGHT / 8.0;
	int width = 60, height = 60;

	for (int y = 0; y < 8; y++) {
		for (int x = 0; x < 8; x++) {
			cMemory::field[x][y].normalColor = CLR_DARKGREEN;
			cMemory::field[x][y].hoverColor = CLR_ORANGE;
			cMemory::field[x][y].selectedColor = CLR_DARKBLUE;
			
			cMemory::field[x][y].rect.x = 200 + (x * xStep + xStep * .5) - width * .5;
			cMemory::field[x][y].rect.y = (y * yStep + yStep * .5) - height * .5;
			cMemory::field[x][y].rect.w = width;
			cMemory::field[x][y].rect.h = height;

			cMemory::field[x][y].enabled = false;
			cMemory::field[x][y].status = CARD_NOTSEL;
		}
	}
}

void cMemory::startMemory(int mode) {
	// Zorgen dat renderMemory() goed functioneerd en een nieuw
	// potje Memory starten.

	cMemory::menuStatus = false;
	cMemory::cardCount = 0;
	
	// Duidelijk maken dat er nog geen eerste of tweede kaartje is aangegeven,
	// en dat de linkermuisknop nog niet is ingedrukt.
	cMemory::firstCard = -1;
	cMemory::secondCard = -1;
	cMemory::leftButton = false;
	cMemory::firstSel.x = -1;
	cMemory::firstSel.y = -1;
	cMemory::secondSel.x = -1;
	cMemory::secondSel.y = -1;
	
	// Scores resetten
	cMemory::p1 = 0;
	cMemory::p2 = 0;
	cMemory::ok = false;
	
	// Bepalen wat er gebeurt bij welke modus. Als de modus GAME_EASY is,
	// achter elk kaartje een PIC_CUSTARD stoppen, zodat je alleen maar
	// dezelfde kunt hebben :P

	// GAME_NORMAL is gewoon normaal memory.
	// GAME_HARD dan zit er achter elk plaatje een willekeurig plaatje ;D

	int tVar = 0;
	int tPos = 0;

	Vector2D vField[64];
	bool bField[64];

	switch (mode) {
		case GAME_EASY:
			for (int y = 0; y < 8; y++) {
				for (int x = 0; x < 8; x++) {
					// cMemory::field[x][y].contains = getIndex(x, y, 8) % 32;
					// cMemory::field[x][y].enabled = true;

					vField[tVar].x = x;
					vField[tVar].y = y;
					bField[tVar] = false;
					tVar++;
				}
			}

			for (int i = 0; i < 32; i++) {
				tPos = rand() % 64;
				while (bField[tPos] == true) {
					tPos = rand() % 64;
				}
				bField[tPos] = true;
				cMemory::field[vField[tPos].x][vField[tPos].y].contains = PIC_TROLL;
				cMemory::field[vField[tPos].x][vField[tPos].y].enabled = true;
				cMemory::field[vField[tPos].x][vField[tPos].y].status = CARD_NOTSEL;

				tPos = rand() % 64;
				while (bField[tPos] == true) {
					tPos = rand() % 64;
				}
				bField[tPos] = true;
				cMemory::field[vField[tPos].x][vField[tPos].y].contains = PIC_CUSTARD;
				cMemory::field[vField[tPos].x][vField[tPos].y].enabled = true;
				cMemory::field[vField[tPos].x][vField[tPos].y].status = CARD_NOTSEL;
			}

			cMemory::status = MEM_P1CARD1;
			break;
		case GAME_NORMAL:
			for (int y = 0; y < 8; y++) {
				for (int x = 0; x < 8; x++) {
					// cMemory::field[x][y].contains = getIndex(x, y, 8) % 32;
					// cMemory::field[x][y].enabled = true;

					vField[tVar].x = x;
					vField[tVar].y = y;
					bField[tVar] = false;
					tVar++;
				}
			}

			for (int i = 0; i < 32; i++) {
				tPos = rand() % 64;
				while (bField[tPos] == true) {
					tPos = rand() % 64;
				}
				bField[tPos] = true;
				cMemory::field[vField[tPos].x][vField[tPos].y].contains = PIC_CUSTARD;
				cMemory::field[vField[tPos].x][vField[tPos].y].enabled = true;
				cMemory::field[vField[tPos].x][vField[tPos].y].status = CARD_NOTSEL;

				tPos = rand() % 64;
				while (bField[tPos] == true) {
					tPos = rand() % 64;
				}
				bField[tPos] = true;
				cMemory::field[vField[tPos].x][vField[tPos].y].contains = PIC_CUSTARD;
				cMemory::field[vField[tPos].x][vField[tPos].y].enabled = true;
				cMemory::field[vField[tPos].x][vField[tPos].y].status = CARD_NOTSEL;
			}

			cMemory::status = MEM_P1CARD1;
			break;
		case GAME_HARD:
			for (int y = 0; y < 8; y++) {
				for (int x = 0; x < 8; x++) {
					cMemory::field[x][y].contains = PIC_RANDOM;
					cMemory::field[x][y].enabled = true;
				}
			}
			cMemory::status = MEM_P1CARD1;
			break;
	}
}

int cMemory::renderMemory(int mX, int mY, Uint8 mouseState) {
	// De vorige status van de linkermuisknop opslaan om te kunnen checken
	// of er een SDL_BUTTONDOWN was
	cMemory::leftButtonOld = cMemory::leftButton;
	cMemory::leftButton = mouseState & SDL_BUTTON(SDL_BUTTON_LEFT);
	
	// Als het ingame-menu er is, dan de kaarten niet aanklikbaar maken
	if (cMemory::menuStatus) {cMemory::leftButtonOld = true;}

	// String voor de status (linksonder)
	std::string turnStatus = "Aha!";

	// Achtergrond van kaartjes
	degModc = cardBackground(degModc, cMemory::menuStatus);
	applySurface(180, 0, cardBackgroundSurf);
	
	// Als het spel niet tussen 2 beurten in zit, gewoon een kaartje laten kiezen.
	// Anders Duidelijk aangeven waar de plaatjes zaten en daarna weer doorgaan.
	if (cMemory::status != MEM_BREAK1 && cMemory::status != MEM_BREAK2) {
		for (int y = 0; y < 8; y++) {
			for (int x = 0; x < 8; x++) {
				if (cMemory::field[x][y].render(mX, mY, cMemory::leftButton & !(cMemory::leftButtonOld)) == BUTTON_CLICKD) {
					switch (cMemory::status) {
						case MEM_P1CARD1:
							if (cMemory::field[x][y].contains < PIC_RANDOM) {
								cMemory::firstCard = cMemory::field[x][y].contains;
							} else {
								cMemory::firstCard = rand() % 32;
							}
							
							cMemory::firstSel.x = x;
							cMemory::firstSel.y = y;
							cMemory::status = MEM_P1CARD2;
							break;
						case MEM_P1CARD2:
							if (cMemory::field[x][y].contains < PIC_RANDOM) {
								cMemory::secondCard = cMemory::field[x][y].contains;
							} else {
								cMemory::secondCard = (rand() % 31) + 1;
							}

							cMemory::secondSel.x = x;
							cMemory::secondSel.y = y;
							cMemory::status = MEM_BREAK1;
							cMemory::ok = CARD_X;

							if (cMemory::firstCard == cMemory::secondCard) {
								if (cMemory::firstCard == PIC_CUSTARD) {
									cMemory::p1 += 5;
								} else if (cMemory::firstCard == PIC_TROLL) {
									cMemory::p1 -= 5;
								} else {
									cMemory::p1 += 1;
								}
								cMemory::ok = CARD_V;
								cMemory::cardCount++;
							}
							break;
						case MEM_P2CARD1:
							if (cMemory::field[x][y].contains < PIC_RANDOM) {
								cMemory::firstCard = cMemory::field[x][y].contains;
							} else {
								cMemory::firstCard = rand() % 32;
							}
							
							cMemory::firstSel.x = x;
							cMemory::firstSel.y = y;
							cMemory::status = MEM_P2CARD2;
							break;
						case MEM_P2CARD2:
							if (cMemory::field[x][y].contains < PIC_RANDOM) {
								cMemory::secondCard = cMemory::field[x][y].contains;
							} else {
								cMemory::secondCard = (rand() % 31) + 1;
							}

							cMemory::secondSel.x = x;
							cMemory::secondSel.y = y;
							cMemory::status = MEM_BREAK2;
							cMemory::ok = CARD_X;

							if (cMemory::firstCard == cMemory::secondCard) {
								if (cMemory::firstCard == PIC_CUSTARD) {
									cMemory::p2 += 5;
								} else if (cMemory::firstCard == PIC_TROLL) {
									cMemory::p2 -= 5;
								} else {
									cMemory::p2++;
								}
								cMemory::ok = CARD_V;
								cMemory::cardCount++;
							}
							break;
					}
				}
			}
		}
	} else if (cMemory::status == MEM_BREAK1) {
		for (int y = 0; y < 8; y++) {
			for (int x = 0; x < 8; x++) {
				cMemory::field[x][y].render(mX, mY, false);
			}
		}

		if (cMemory::ok == CARD_V) {
			if (cMemory::firstCard == PIC_TROLL) {
				turnStatus = "Player two's turn!";
			} else {
				turnStatus = "Player one's turn!";
			}
		} else if (cMemory::ok == CARD_X) {
			turnStatus = "Player two's turn!";
		}
	} else if (cMemory::status == MEM_BREAK2) {
		for (int y = 0; y < 8; y++) {
			for (int x = 0; x < 8; x++) {
				cMemory::field[x][y].render(mX, mY, false);
			}
		}
		
		if (cMemory::ok == CARD_V) {
			if (cMemory::firstCard == PIC_TROLL) {
				turnStatus = "Player one's turn!";
			} else {
				turnStatus = "Player two's turn!";
			}
		} else if (cMemory::ok == CARD_X) {
			turnStatus = "Player one's turn!";
		}
	}
	
	// De gekozen plaatjes weergeven.
	if (cMemory::firstCard >= 0) {
		applySurfaceMid(90, 135, memPics[cMemory::firstCard], screen);
	}
	if (cMemory::secondCard >= 0) {
		applySurfaceMid(90, 350, memPics[cMemory::secondCard], screen);
	}
	
	// Een 1 plaatsen op het eerst gekozen plaatje en een 2 op het tweede plaatje.
	float step = 600 / 8.0;
	if (cMemory::firstSel.x != -1) {
		applySurfaceMid(200 + cMemory::firstSel.x * step + step * .5, cMemory::firstSel.y * step + step * .5, one, screen);
	}

	if (cMemory::secondSel.x != -1) {
		applySurfaceMid(200 + cMemory::secondSel.x * step + step * .5, cMemory::secondSel.y * step + step * .5, two, screen);
	}
	
	// Een 1 plaatsen bij het bovenste plaatje en een 2 bij het onderste.
	applySurfaceMid(90, 25, firstPic, screen);
	applySurfaceMid(90, 240, secondPic, screen);

	// Score weergeven
	displayScore (cMemory::p1, cMemory::p2);

	// Randen
	/*
	balk(0, 0, 180, 10, HSLtoRGB(awesomeColor, 0.5, 0.5)); // Boven Balk
	balk(180, 0, 10, 453, HSLtoRGB(awesomeColor, 0.5, 0.5)); // RechterZij Balk
	balk(180, 453, 10, 147, HSLtoRGB(awesomeColor, 0.5, 0.5)); // MiddenBoven Balk
	balk(0, 479, 180, 10, HSLtoRGB(awesomeColor, 0.5, 0.5)); // MiddenOnder Balk
	balk(0, 520, 180, 5, HSLtoRGB(awesomeColor, 0.5, 0.5));// Scoreverdelingsbalken
	balk(88, 525, 5, 65, HSLtoRGB(awesomeColor, 0.5, 0.5));
	balk(70, 488, 10, 35, HSLtoRGB(awesomeColor, 0.5, 0.5)); // Menu knop tussenbalk
	balk(0, 590, 180, 10, HSLtoRGB(awesomeColor, 0.5, 0.5)); // Onderbalk
	*/

	// SDL_MapRGB(screen->format, 0, 191, 255);

	balk(0, 0, 180, 10, SDL_MapRGB(screen->format, 0, 191, 255)); // Boven Balk
	// balk(180, 0, 10, 453, SDL_MapRGB(screen->format, 0, 191, 255)); // RechterZij Balk
	balk(180, 453, 10, 147, SDL_MapRGB(screen->format, 0, 191, 255)); // MiddenBoven Balk
	balk(0, 479, 180, 10, SDL_MapRGB(screen->format, 0, 191, 255)); // MiddenOnder Balk
	balk(0, 520, 180, 5, SDL_MapRGB(screen->format, 0, 191, 255));// Scoreverdelingsbalken
	balk(88, 525, 5, 65, SDL_MapRGB(screen->format, 0, 191, 255));
	balk(70, 488, 10, 35, SDL_MapRGB(screen->format, 0, 191, 255)); // Menu knop tussenbalk
	balk(0, 590, 180, 10, SDL_MapRGB(screen->format, 0, 191, 255)); // Onderbalk

	applySurface(0, 0, cardBGRanden);


	// Check voor eind
	if (cMemory::cardCount == 32) {
		if (cMemory::p1 > cMemory::p2) {
			applySurface(190, 0, p1Wins);
		} else if (cMemory::p2 > cMemory::p1) {
			applySurface(190, 0, p2Wins);
		} else if (cMemory::p1 == cMemory::p2) {
			applySurface(190, 0, tied);
		}
	}
	
	// Statusdingetje & caption daarvoor
	switch (cMemory::status) {
		case MEM_P1CARD1:
			turnStatus = "P1 chooses a card";
			break;
		case MEM_P1CARD2:
			turnStatus = "P1 chooses a card";
			break;
		case MEM_P2CARD1:
			turnStatus = "P2 chooses a card";
			break;
		case MEM_P2CARD2:
			turnStatus = "P2 chooses a card";
			break;
	}

	// Knop die ervoor zorgt dat je doorgaat naar de volgende beurt als er 2 kaarten gekozen zijn.
	if (button(turnStatus.c_str(), 180, 454, mX, mY, cMemory::leftButton & !(cMemory::leftButtonOld), font2, cMemory::leftButton) == BUTTON_CLICKD) {
		if (cMemory::status == MEM_BREAK1) {			
			if (cMemory::ok == CARD_V) {
				cMemory::field[cMemory::firstSel.x][cMemory::firstSel.y].enabled = false;
				cMemory::field[cMemory::secondSel.x][cMemory::secondSel.y].enabled = false;

				if (cMemory::firstCard == PIC_TROLL) {
					cMemory::status = MEM_P2CARD1;
				} else {
					cMemory::status = MEM_P1CARD1;
				}
			} else {
				cMemory::field[cMemory::firstSel.x][cMemory::firstSel.y].status = CARD_NOTSEL;
				cMemory::field[cMemory::secondSel.x][cMemory::secondSel.y].status = CARD_NOTSEL;
				cMemory::status = MEM_P2CARD1;
			}

			cMemory::ok = CARD_NONE;

			cMemory::firstCard = -1;
			cMemory::secondCard = -1;

			cMemory::firstSel.x = -1;
			cMemory::firstSel.y = -1;

			cMemory::secondSel.x = -1;
			cMemory::secondSel.y = -1;
		} else if (cMemory::status == MEM_BREAK2) {
			if (cMemory::ok == CARD_V) {
				cMemory::field[cMemory::firstSel.x][cMemory::firstSel.y].enabled = false;
				cMemory::field[cMemory::secondSel.x][cMemory::secondSel.y].enabled = false;

				if (cMemory::firstCard == PIC_TROLL) {
					cMemory::status = MEM_P1CARD1;
				} else {
					cMemory::status = MEM_P2CARD1;
				}
			} else {
				cMemory::field[cMemory::firstSel.x][cMemory::firstSel.y].status = CARD_NOTSEL;
				cMemory::field[cMemory::secondSel.x][cMemory::secondSel.y].status = CARD_NOTSEL;
				cMemory::status = MEM_P1CARD1;
			}

			cMemory::ok = CARD_NONE;

			cMemory::firstCard = -1;
			cMemory::secondCard = -1;

			cMemory::firstSel.x = -1;
			cMemory::firstSel.y = -1;

			cMemory::secondSel.x = -1;
			cMemory::secondSel.y = -1;
		}
	}

	// V'tje of X'je plaatsen & Pijltjes voor doorgaan
	if (cMemory::ok == CARD_V) {
		applySurfaceMid(90, 227, v, screen);
		applySurface(1, 422, arrows);
	} else if (cMemory::ok == CARD_X) {
		applySurfaceMid(90, 227, x, screen);
		applySurface(1, 422, arrows);
	}
	
	// Ingame-menu triggering & logica
	if (button("Menu", 58, 494, mX, mY, cMemory::leftButton & !(cMemory::leftButtonOld), font2, cMemory::leftButton) == BUTTON_CLICKD) {
		cMemory::menuStatus = true;
		
		bResume.menuState = STATE_IN;
		bBack_Menu.menuState = STATE_IN;
		bExit_Game.menuState = STATE_IN;
	}
	
	float tA = bResume.area.x + bResume.margin + BUTTON_WIDTH * .5;
	float tB = tA - SCR_WIDTH / (float)2;
	float tC = tB / (float)(SCR_WIDTH / (float)2);
	applySurfaceMid(tA + tC * (float)295, 125, paused, screen);

	bResume.render(mouseState, mX, mY, &buttonResume);
	bBack_Menu.render(mouseState, mX, mY, &buttonBack_Menu);
	bExit_Game.render(mouseState, mX, mY, &buttonExit_Game);

	return 0;
}

cCard::cCard() {
	// Geen idee eigenlijk
	cCard::enabled = true;
}

int cCard::render(int mX, int mY, bool button) {
	// The waarde die teruggegeven wordt, afhankelijk van
	// of er geklikt is of niet.
	int returnVal;

	// Coordinaten voor de plaatjes of het spritesheet
	SDL_Rect rects[3];
	for (int i = 0; i < 3; i++) {
		rects[i].w = 60;
		rects[i].h = 60;
		rects[i].x = i * 60;
		rects[i].y = 0;
	}
	
	if (!cCard::enabled) {
		return BUTTON_NOT;
	}
	
	// Als de kaart al geselecteerd is, dan alleen tekenen en 'm weer peren.
	if (cCard::status == CARD_SELECTED) {
		// SDL_FillRect(screen, &(cCard::rect), cCard::selectedColor);
		SDL_BlitSurface(kaartje, &rects[2], screen, &(cCard::rect));
		return BUTTON_NOT;
	}
	
	
	if (mX >= cCard::rect.x && mX <= cCard::rect.x + cCard::rect.w) {			//
		if (mY >= cCard::rect.y && mY <= cCard::rect.y + cCard::rect.h) {		//
			if (button) {														//
				// SDL_FillRect(screen, &(cCard::rect), cCard::selectedColor);		// Als de muis in een gebiedje is en er wordt geklikt,
				SDL_BlitSurface(kaartje, &rects[2], screen, &(cCard::rect));
				cCard::status = CARD_SELECTED;									// Dan goede waarde teruggeven en kaartstatus veranderen.
				returnVal = BUTTON_CLICKD;										//
			} else {															//
				// SDL_FillRect(screen, &(cCard::rect), cCard::hoverColor);		// Hetzelfde ^^
				SDL_BlitSurface(kaartje, &rects[1], screen, &(cCard::rect));
				returnVal = BUTTON_OVER;										//
			}																	//
		} else {																//
			// SDL_FillRect(screen, &(cCard::rect), cCard::normalColor);			// Hetzelfde ^^
			SDL_BlitSurface(kaartje, &rects[0], screen, &(cCard::rect));
			returnVal = BUTTON_NOT;												//
		}																		//
	} else {																	//
		// SDL_FillRect(screen, &(cCard::rect), cCard::normalColor);				// Wat denk je?
		SDL_BlitSurface(kaartje, &rects[0], screen, &(cCard::rect));
		returnVal = BUTTON_NOT;													//
	}																			//
	
	// Zorgen dat de kaart een CARD_SELECTED kleur krijgt als hij geselecteerd is,
	// ook al is er verder niks met de knop.
	if (cCard::status == CARD_SELECTED) {
		// SDL_FillRect(screen, &(cCard::rect), cCard::selectedColor);
		SDL_BlitSurface(kaartje, &rects[2], screen, &(cCard::rect));
	}

	return returnVal;
}